/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#include <stdlib.h>
#include <string.h>
#include "CzMemStream.h"

static int CzMemStream_getch(CZSTREAM *stream);
static int CzMemStream_ungetc(CZSTREAM *stream, int c);
static int CzMemStream_eof(CZSTREAM *stream);
static int CzMemStream_close(CZSTREAM *stream);

/* メモリストリームの初期化 */
void CzMemStream_init(CZMEMSTREAM *stream, char *data, size_t size)
{
	CzStream_init(&stream->super);
	stream->super.getch = CzMemStream_getch;
	stream->super.ungetc = CzMemStream_ungetc;
	stream->super.eof = CzMemStream_eof;
	stream->super.close = CzMemStream_close;
	stream->buffer = (char*)malloc(size);
	memcpy(stream->buffer, data, size);
	stream->pt = 0;
	stream->size = size;
	stream->unget_c_flag = FALSE;
}

/* １文字取得 */
static int CzMemStream_getch(CZSTREAM *stream)
{
	int ret;
	CZMEMSTREAM *st = (CZMEMSTREAM *)stream;
	if(st->unget_c_flag) { /* １文字戻されているなら */
		/* 戻されている１文字を返す */
		st->unget_c_flag = FALSE;
		return st->unget_c;
	}
	/* 初期サイズ以上なら */
	if(st->pt >= st->size) {
		/* 終端を返す */
		return EOF;
	}
	ret = st->buffer[st->pt];
	st->pt++;
	return ret;
}

/* １文字戻す */
static int CzMemStream_ungetc(CZSTREAM *stream, int c)
{
	CZMEMSTREAM *st = (CZMEMSTREAM *)stream;
	st->unget_c = c;
	st->unget_c_flag = TRUE;
	return c;
}

/* 終端チェック */
static int CzMemStream_eof(CZSTREAM *stream)
{
	CZMEMSTREAM *st = (CZMEMSTREAM *)stream;
	if(st->unget_c_flag) {
		return 0;
	}
	if(st->pt >= st->size) {
		return 1;
	}
	return 0;
}

/* ストリームを閉じる */
static int CzMemStream_close(CZSTREAM *stream)
{
	CZMEMSTREAM *st = (CZMEMSTREAM *)stream;
	free(st->buffer);
	return 1; 
}
