/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#include <string.h>
#include "CzMemory.h"

/* CZMEMORYの初期化 */
void CzMemory_init(CZMEMORY *mem)
{
	mem->data = NULL;
	mem->size = 0;
	mem->mem_size = 0;
}

/* CZMEMORYの開放 */
void CzMemory_delete(CZMEMORY *mem)
{
	if(mem->data) {
		free(mem->data);
	}
	CzMemory_init(mem);
}

/* CZMEMORYへデータ追加 */
BOOL CzMemory_add_data(CZMEMORY *mem, char data)
{
	void *tmp;

	if(mem->size == mem->mem_size) {
		tmp = realloc(mem->data, mem->mem_size + ALLOC_MEM_SIZE);
		if(!tmp) {
			return FALSE;
		}
		mem->data = tmp;
		mem->mem_size += ALLOC_MEM_SIZE;
	}
	mem->data[mem->size] = data;
	mem->size++;

	return TRUE;
}

BOOL CzMemory_add_datas(CZMEMORY *mem, char *data, size_t size)
{
	void *tmp;
	size_t tmpsize = mem->size + size;

	if(tmpsize > mem->mem_size) {
		size_t new_size = (tmpsize / ALLOC_MEM_SIZE + 1) * ALLOC_MEM_SIZE;
		tmp = realloc(mem->data, new_size);
		if(!tmp) {
			return FALSE;
		}
		mem->data = tmp;
		mem->mem_size = new_size;
	}
	memcpy(&mem->data[mem->size], data, size);
	mem->size += size;

	return TRUE;
}
