/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#ifndef __CZSTRING_H__
#define __CZSTRING_H__

#include "CzMemory.h"
#ifdef HAVE_ICONV
#include <iconv.h>
#endif

typedef CZMEMORY CZSTRING;
typedef wchar_t CZCHAR;
typedef UINT16 CZUTF16CHAR;

/* CZSTRINGの初期化 */
void CzString_init(CZSTRING *string);

/* CZSTRINGのメモリ開放 */
void CzString_delete(CZSTRING *string);

/* CZSTRINGへ文字列追加 */
#define CzString_add_data(mem, data) CzMemory_add_data(mem, data)
#define CzString_add_datas(mem, data, size) CzMemory_add_datas(mem, data, size)
/* 文字列の追加 */
BOOL CzString_add_string(CZSTRING *str1, CZSTRING *str2);
/* 文字の追加 */
BOOL CzString_add_char(CZSTRING *str, CZCHAR ch);
/* CZSTRINGのコピー */
BOOL CzString_copy(CZSTRING *dest, CZSTRING *src);

#ifdef HAVE_ICONV
typedef struct czencoding {
	iconv_t handle_iconv;
} CZENCODING;

/* CZENCODINGの初期化 */
void CzEncoding_init(CZENCODING *enc, const char *tocode, const char *fromcode);

/* CZENCODINGの開放 */
void CzEncoding_delete(CZENCODING *enc);

#else
typedef enum convert_mode {
	CONVERT_MODE_MULTBYTE_TO_UTF16,
	CONVERT_MODE_UTF16_TO_MULTBYTE,
	CONVERT_MODE_UTF8_TO_UTF16,
	CONVERT_MODE_UTF16_TO_UTF8,
} CONVERT_MODE;

typedef struct czencoding {
	CONVERT_MODE mode;
} CZENCODING;

/* UTF32->UTF16変換 */
BOOL CzEncoding_convert_UTF32_to_UTF16(CZSTRING *dest, CZSTRING *src);
/* UTF16->UTF32変換 */
BOOL CzEncoding_convert_UTF16_to_UTF32(CZSTRING *dest, CZSTRING *src);
#endif
/* 文字列の文字コード変換 */
BOOL CzEncoding_convert(CZENCODING *enc, CZSTRING *dest, CZSTRING *src);

#endif
