/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#ifndef __LEX_ANALYZER_H__
#define __LEX_ANALYZER_H__

#include <stdio.h>
#include "CzStream.h"
#include "CzarBasic.h"
#include "CzarBasic_error.h"
#include "CzMemory.h"

#define STRING_SIZE	256

/* Tokenの種類 */
enum _token_type {
	TOKEN_TYPE_NONE,
	TOKEN_TYPE_ERROR,
	TOKEN_TYPE_EOF,
	TOKEN_TYPE_NL,		/* 改行 */
	TOKEN_TYPE_WORD,	/* 単語 */
	TOKEN_TYPE_SPACE,	/* スペース */
	TOKEN_TYPE_MARK,	/* 記号 */
	TOKEN_TYPE_STRING,	/* 文字列 */
	TOKEN_TYPE_LPARENTHESES,	/* ( */
	TOKEN_TYPE_RPARENTHESES,	/* ) */
	TOKEN_TYPE_PLUS,		/* + */
	TOKEN_TYPE_MINUS,		/* - */
	TOKEN_TYPE_MUL,		/* * */
	TOKEN_TYPE_DIV,		/* / */
	TOKEN_TYPE_EQ,		/* = */
	TOKEN_TYPE_GT,		/* > */
	TOKEN_TYPE_GE,		/* >= */
	TOKEN_TYPE_LT,		/* < */
	TOKEN_TYPE_LE,		/* <= */
	TOKEN_TYPE_NE,		/* <> */
	TOKEN_TYPE_COLON,		/* : */
	TOKEN_TYPE_COMMA,		/* , */
	TOKEN_TYPE_SEMICOLON,		/* ; */

	TOKEN_TYPE_PRINT,	/* PRINT */
	TOKEN_TYPE_IF,		/* IF */
	TOKEN_TYPE_GOTO,	/* GOTO */
	TOKEN_TYPE_LET,		/* LET */
	TOKEN_TYPE_INPUT,	/* INPUT */
	TOKEN_TYPE_GOSUB,	/* GOSUB */
	TOKEN_TYPE_RETURN,	/* RETURN */
	TOKEN_TYPE_CLEAR,	/* CLEAR */
	TOKEN_TYPE_END,		/* END */
	TOKEN_TYPE_THEN,	/* THEN */
	TOKEN_TYPE_REM,		/* REM */
};
typedef enum _token_type TOKEN_TYPE;

/* TOKEN */
typedef struct token {
	TOKEN_TYPE token_type;		/* Tokenの種類 */
	char token[STRING_SIZE];	/* Token */
} TOKEN;

/* lexのデータ */
typedef struct lex_data {
	CZSTREAM *stream;
	ERROR_CODE err_code;		/* エラーコード */
	int err_line_num;	/* エラー時の行番号 */
	CZMEMORY label_address;		/* ラベルのアドレスを埋め込むアドレスリスト */
} LEX_DATA;

/* LEX_DATAの初期化 */
void lex_data_init(LEX_DATA *lex_data, CZSTREAM *stream);
/* LEX_DATAのメモリの開放 */
void lex_data_delete(LEX_DATA *lex_data);

/* Tokenを取得する */
TOKEN *lex_get_token(LEX_DATA *stream);
/* tokenを戻す */
void lex_unget_token(TOKEN *token);

#ifdef LEX_TEST
int lex_test(int argc, char *argv[]);
#endif

/* 文字列を比較 */
int Strcmp(const char *str1, const char *str2);
#endif
